#include <algorithm>
#include <cstdio>
#include <cstdlib>
#include <list>
#include "findnumber.h"

//bool isInside(long long l, long long r);

// Interval is [0..N)
// 0 <= l, r <= N
// [l, r) iff l < r
// [l, N) U [0, r) iff l > r
// Contestant function.
//
// Store only simple intervals.
//
struct interval {
    long long l, r;
};

int steps = 0;
long long N, x;
long long flip;
// Single interval assuming everything is true.
//
std::list<interval> A;
// The set of intervals where the answer might be. Disjoint.
//
std::list<interval> B;

interval inters_simple(interval a, long long l, long long r) {
    interval res;
    res.l = std::max(a.l, l);
    res.r = std::min(a.r, r);
    if (res.l >= res.r) {
        res.l = res.r = -1;
    }
    return res;
}

// Remove [l, r) from a - simple intervals.
//
std::list<interval> sub_simple(interval a, long long l, long long r) {
    std::list<interval> res;
    if (a.l < l) {
        res.push_back({a.l, std::min(l, a.r)});
    }
    if (r < a.r) {
        res.push_back({std::max(a.l, r), a.r});
    }
    return res;
}

void intersect(const std::list<interval> &s, long long l, long long r,
        std::list<interval> &common, std::list<interval> &disjoint) {

    if (l == r) {
        disjoint = s;
        return;
    }

    for (std::list<interval>::const_iterator it = s.begin(); it != s.end(); ++it) {
        if (l < r) {
            interval com = inters_simple(*it, l, r);
            if (com.l != -1) {
                common.push_back(com);
            }
            disjoint.splice(disjoint.begin(), sub_simple(*it, l, r));
        } else if (l > r) {
            // Spawn at most one extra among all here.
            //
            std::list<interval> dis;
            interval com = inters_simple(*it, l, N);
            if (com.l != -1) {
                common.push_back(com);
            }
            dis = sub_simple(*it, l, N);
            com = inters_simple(*it, 0, r);
            if (com.l != -1) {
                common.push_back(com);
            }
            for (std::list<interval>::const_iterator jt = dis.begin();
                    jt != dis.end(); ++jt) {
                disjoint.splice(disjoint.begin(), sub_simple(*jt, 0, r));
            }
        }
    }
}

long long set_size(const std::list<interval> &s) {
    long long sz = 0;
    for (std::list<interval>::const_iterator it = s.begin(); it != s.end(); ++it) {
        sz += it->r - it->l;
    }
    return sz;
}

long long eval(long long a, long long b) {
    if (a == 1 && b == 0) {
        return 0;
    }
    if (a == 0 && b == 1) {
        return 0;
    }
    return 1;
}

// Methods available to contestant
//

// Ignoring x and flip.
//
long long bsize(long long l, long long r) {
    std::list<interval> B1, B2;
    intersect(B, l, r, B1, B2);
    return set_size(B1);
}

void update_state(long long l, long long r, bool res) {
    std::list<interval> A1, A2, B1, B2;
    intersect(A, l, r, A1, A2);
    intersect(B, l, r, B1, B2);

    if (res) {
        A = A1;
        B = A2;
        B.splice(B.begin(), B2);
    } else {
        A = A2;
        B = A1;
        B.splice(B.begin(), B1);
    }
}

void pstate(const std::list<interval> &s) {
    for (std::list<interval>::const_iterator it = s.begin(); it != s.end(); ++it) {
        printf("[%lld, %lld) ", it->l, it->r);
    }
    printf("\n");
}

long long play(long long n) {
    N = n;
    A.push_back({0, n});
    long long k;
    while ((k = eval(set_size(A), set_size(B))) > 0) {
        // printf("STATE IN SOLUTION -> %lld %lld %lld\n", k, set_size(A), set_size(B));
        // pstate(A);
        // pstate(B);
        // printf("\n");
        // Size of set A is one!
        //
        long long qa = set_size(A) / 2;
        long long qb;
        qb = set_size(B) / 2;
        if (qb < 0) {
            qb = 0;
        } else if (qb > set_size(B)) {
            qb = set_size(B);
        }
        // qb = set_size(B) - qb;
        // qa = set_size(A) - qa;
        if (qa == 0 && qb == 0) {
            if (set_size(B)) {
                qb = set_size(B) - qb;
            }
            if (set_size(A)) {
                qa = set_size(A) - qa;
            }
        }
        long long ql;
        if (A.size()) {
            ql = A.begin()->r - qa;
        } else {
            ql = 0;
        }
        long long szl, szr;
        szl = qa + qb - 1; szr = N;
        while (szl + 1 < szr) {
            long long szmid = (szl + szr) / 2;
            if (bsize(ql, (ql + szmid) % N) >= qb) {
                szr = szmid;
            } else {
                szl = szmid;
            }
        }
        long long qr = (ql + szr) % N;
        // printf("OPA %lld %lld (%lld) -> %lld %lld\n", set_size(A), set_size(B), k, qa, qb);
        bool res = isInside(ql, qr);
        update_state(ql, qr, res);
    }
    // printf("FIN %lld %lld -> %lld\n", set_size(A), set_size(B), k);
    if (set_size(A)) {
        return (A.begin()->l);
    } else {
        return (B.begin()->l);
    }
}

