#include <algorithm>
#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <list>
#include "findnumber.h"
using namespace std;

//bool isInside(long long l, long long r);

// Interval is [0..N)
// 0 <= l, r <= N
// [l, r) iff l < r
// [l, N) U [0, r) iff l > r
// Contestant function.
//
// Store only simple intervals.
//
struct interval {
    long long l, r;
};

int steps = 0;
long long N, x;
long long flip;
// Single interval assuming everything is true.
//
std::list<interval> A;
// The set of intervals where the answer might be. Disjoint.
//
std::list<interval> B;

interval inters_simple(interval a, long long l, long long r) {
    interval res;
    res.l = std::max(a.l, l);
    res.r = std::min(a.r, r);
    if (res.l >= res.r) {
        res.l = res.r = -1;
    }
    return res;
}

// Remove [l, r) from a - simple intervals.
//
std::list<interval> sub_simple(interval a, long long l, long long r) {
    std::list<interval> res;
    if (a.l < l) {
        res.push_back({a.l, std::min(l, a.r)});
    }
    if (r < a.r) {
        res.push_back({std::max(a.l, r), a.r});
    }
    return res;
}

void intersect(const std::list<interval> &s, long long l, long long r,
        std::list<interval> &common, std::list<interval> &disjoint) {

    if (l == r) {
        disjoint = s;
        return;
    }

    for (std::list<interval>::const_iterator it = s.begin(); it != s.end(); ++it) {
        if (l < r) {
            interval com = inters_simple(*it, l, r);
            if (com.l != -1) {
                common.push_back(com);
            }
            disjoint.splice(disjoint.begin(), sub_simple(*it, l, r));
        } else if (l > r) {
            // Spawn at most one extra among all here.
            //
            std::list<interval> dis;
            interval com = inters_simple(*it, l, N);
            if (com.l != -1) {
                common.push_back(com);
            }
            dis = sub_simple(*it, l, N);
            com = inters_simple(*it, 0, r);
            if (com.l != -1) {
                common.push_back(com);
            }
            for (std::list<interval>::const_iterator jt = dis.begin();
                    jt != dis.end(); ++jt) {
                disjoint.splice(disjoint.begin(), sub_simple(*jt, 0, r));
            }
        }
    }
}

long long set_size(const std::list<interval> &s) {
    long long sz = 0;
    for (std::list<interval>::const_iterator it = s.begin(); it != s.end(); ++it) {
        sz += it->r - it->l;
    }
    return sz;
}

long long h[1024][1024];

long long eval(long long a, long long b, int *aa = NULL, int *bb = NULL) {
    if (aa == NULL) {
        if (h[a][b] != -1) {
            return h[a][b];
        }
    }
    // printf("EVAL %lld %lld %p %p %lld\n", a, b, aa, bb, h[a][b]);
    if (!a && !b) {
        return -1;
    }
    if (a == 1 && b == 0) {
        return 0;
    }
    if (a == 0 && b == 1) {
        return 0;
    }
    int a1, a2, b1, b2;
    long long sol = 10000;
    for (int a1 = 0; a1 <= a; ++a1) {
        for (int b1 = 0; b1 <= b; ++b1) {
            a2 = a - a1;
            b2 = b - b1;
            if (a1 == a && a2 + b2 == b) {
                continue;
            }
            if (a2 == a && a1 + b1 == b) {
                continue;
            }
            // if (a1 == 0 && b1 == 0) {
            //     continue;
            // }
            // if (a2 == 0 && b2 == 0) {
            //     continue;
            // }
            // printf("GO %d %d %lld\n", a1, b1, max(eval(a1, a2 + b2), eval(b2, a1 + b1)));
            // printf("GO %lld %lld -> %d %d\n", a, b, a1, b1);
            if (max(eval(a1, a2 + b2), eval(a2, a1 + b1)) + 1 < sol) {
                sol = max(eval(a1, a2 + b2), eval(a2, a1 + b1)) + 1;
                if (aa) {
                    *aa = a1;
                    *bb = b1;
                }
            }
        }
    }
    return (h[a][b] = sol);
}

// Methods available to contestant
//

// Ignoring x and flip.
//
long long bsize(long long l, long long r) {
    std::list<interval> B1, B2;
    intersect(B, l, r, B1, B2);
    return set_size(B1);
}

void update_state(long long l, long long r, bool res) {
    std::list<interval> A1, A2, B1, B2;
    intersect(A, l, r, A1, A2);
    intersect(B, l, r, B1, B2);

    if (res) {
        A = A1;
        B = A2;
        B.splice(B.begin(), B2);
    } else {
        A = A2;
        B = A1;
        B.splice(B.begin(), B1);
    }
}

void pstate(const std::list<interval> &s) {
    for (std::list<interval>::const_iterator it = s.begin(); it != s.end(); ++it) {
        printf("[%lld, %lld) ", it->l, it->r);
    }
    printf("\n");
}

long long play(long long n) {
    memset(h, -1, sizeof(h));
    N = n;
    A.push_back({0, n});
    long long k;
    int a1, b1;
    while ((k = eval(set_size(A), set_size(B), &a1, &b1)) > 0) {
        // Size of set A is one!
        //
        long long qa = a1;
        long long qb = b1;
        if (qb < 0) {
            qb = 0;
        } else if (qb > set_size(B)) {
            qb = set_size(B);
        }
        if (qa == 0 && qb == 0) {
            if (set_size(B)) {
                qb = set_size(B) - qb;
            }
            if (set_size(A)) {
                qa = set_size(A) - qa;
            }
        }
        long long ql;
        if (A.size()) {
            ql = A.begin()->r - qa;
        } else {
            ql = 0;
        }
        long long szl, szr;
        szl = qa + qb - 1; szr = N;
        while (szl + 1 < szr) {
            long long szmid = (szl + szr) / 2;
            if (bsize(ql, (ql + szmid) % N) >= qb) {
                szr = szmid;
            } else {
                szl = szmid;
            }
        }
        long long qr = (ql + szr) % N;
        bool res = isInside(ql, qr);
        update_state(ql, qr, res);
    }
    if (set_size(A)) {
        return (A.begin()->l);
    } else {
        return (B.begin()->l);
    }
}

