/*
ID: espr1t
TASK: MatrixSearch
KEYWORDS: Testgen
*/

#include <cstdio>
#include <ctime>
#include <cstring>
#include <queue>
#include <random>
FILE* in = stdin; FILE* out = stdout;

using namespace std;
const int MAX = 1001;

struct Query {
    int type;
    int row, col, val;
    
    Query(int type_, int val_) {
        type = type_, val = val_;
    }
    
    Query(int type_, int row_, int col_, int val_) {
        type = type_, row = row_, col = col_, val = val_;
    }
};

struct Test {
    vector <vector <int>> board;
    vector <Query> queries;
};
vector <Test> tests;


mt19937 mt;
int rnd() {
    int ret = mt();
    return ret < 0 ? -ret : ret;
}

class Author {
    int n, m;
    int a[MAX][MAX];
    int rows[MAX][MAX], cols[MAX][MAX];
    
    public:
        
    void init(Test& test) {
        n = (int)test.board.size(), m = (int)test.board[0].size();
        for (int row = 0; row < n; row++) {
            for (int col = 0; col < m; col++) {
                a[row][col] = test.board[row][col];
                rows[row][col] = cols[row][col] = 0;
            }
        }
    }
    
    bool find(int needle) {
        int row = 0, col = m - 1;
        int add = rows[row][col];
    
        while (row < n && col >= 0) {
            while (row + 1 < n && a[row + 1][col] + add + rows[row + 1][col] <= needle)
                add += rows[++row][col];
            while (col - 1 >= 0 && a[row][col - 1] + add - cols[row][col] >= needle)
                add -= cols[row][col--];
            if (a[row][col] + add == needle)
                return true;
            add -= cols[row][col--];
        }
        return false;
    }
    
    void increment(int srow, int scol, int val) {
        for (int col = scol; col < m; col++)
            rows[srow][col] += val;
        for (int row = srow; row < n; row++)
            cols[row][scol] += val;
    }
};

class Dummy {
    int n, m;
    int a[MAX][MAX];
    
    public:
        
    void init(Test& test) {
        n = (int)test.board.size(), m = (int)test.board[0].size();
        for (int row = 0; row < n; row++) {
            for (int col = 0; col < m; col++) {
                a[row][col] = test.board[row][col];
            }
        }
    }
    
    bool find(int needle) {
        for (int row = 0; row < n; row++) {
            for (int col = 0; col < m; col++) {
                if (a[row][col] == needle)
                    return true;
            }
        }
        return false;
    }
    
    void increment(int srow, int scol, int val) {
        for (int row = srow; row < n; row++) {
            for (int col = scol; col < m; col++) {
                a[row][col] += val;
            }
        }
    }
};


class Slow {
    int n, m;
    int a[MAX][MAX];
    
    public:
        
    void init(Test& test) {
        n = (int)test.board.size(), m = (int)test.board[0].size();
        for (int row = 0; row < n; row++) {
            for (int col = 0; col < m; col++) {
                a[row][col] = test.board[row][col];
            }
        }
    }
    
    bool find(int needle) {
        int row = 0, col = m - 1;
        while (row < n && col >= 0) {
            while (row + 1 < n && a[row + 1][col] <= needle)
                row++;
            while (col - 1 >= 0 && a[row][col - 1] >= needle)
                col--;
            if (a[row][col] == needle)
                return true;
            col--;
        }
        return false;
    }
    
    void increment(int srow, int scol, int val) {
        for (int row = srow; row < n; row++) {
            for (int col = scol; col < m; col++) {
                a[row][col] += val;
            }
        }
    }
};

class Okay {
    int n, m;
    int a[MAX][MAX];
    int tree[MAX][MAX];
    
    public:
        
    void init(Test& test) {
        n = (int)test.board.size(), m = (int)test.board[0].size();
        for (int row = 0; row < n; row++) {
            for (int col = 0; col < m; col++) {
                a[row][col] = test.board[row][col];
            }
        }
        for (int row = 0; row <= n; row++)
            for (int col = 0; col <= m; col++)
                tree[row][col] = 0;
    }
    
    void update(int x, int y, int val) {
        for (int x1 = x + 1; x1 < MAX; x1 += (x1 & -x1))
    	    for (int y1 = y + 1; y1 < MAX; y1 += (y1 & -y1))
    			tree[x1][y1] += val;
    }
    
    int query(int x, int y) {
    	int ret = 0;
    	for (int x1 = x + 1; x1 > 0; x1 -= (x1 & -x1))
    	    for (int y1 = y + 1; y1 > 0; y1 -= (y1 & -y1))
    	        ret += tree[x1][y1];
    	return ret;
    }
    
    int getCell(int row, int col) {
        return a[row][col] + query(row, col);
    }
    
    bool find(int needle) {
        int row = 0, col = m - 1;
        while (row < n && col >= 0) {
            while (row + 1 < n && getCell(row + 1, col) <= needle)
                row++;
            while (col - 1 >= 0 && getCell(row, col - 1) >= needle)
                col--;
            if (getCell(row, col) == needle)
                return true;
            col--;
        }
        return false;
    }
    
    void increment(int srow, int scol, int val) {
        update(srow, scol, val);
    }
};

void testSolutions() {
    // Azshmy
    Dummy dummy;
    Slow slow;
    Okay okay;
    Author author;

    for (int t = 0; t < (int)tests.size(); t++) {
        fprintf(stderr, "Currently solving testcase %d (out of %d)\n", t, (int)tests.size() - 1);
        
        Test& test = tests[t];

        unsigned dTime = clock();
        dummy.init(test);
        string ansDummy;
        for (int i = 0; i < (int)test.queries.size(); i++) {
            if (test.queries[i].type == 1) { // Query
                ansDummy.push_back(dummy.find(test.queries[i].val) ? 'Y' : 'N');
            } else { // Update
                dummy.increment(
                    test.queries[i].row - 1, test.queries[i].col - 1, (int)test.queries[i].val);
            }
        }
        fprintf(stderr, "  >> dummy's time: %.3lfs\n", (double)(clock() - dTime) / CLOCKS_PER_SEC);

        unsigned sTime = clock();
        slow.init(test);
        string ansSlow;
        for (int i = 0; i < (int)test.queries.size(); i++) {
            if (test.queries[i].type == 1) { // Query
                ansSlow.push_back(slow.find(test.queries[i].val) ? 'Y' : 'N');
            } else { // Update
                slow.increment(
                    test.queries[i].row - 1, test.queries[i].col - 1, (int)test.queries[i].val);
            }
        }
        fprintf(stderr, "  >> slow's time: %.3lfs\n", (double)(clock() - sTime) / CLOCKS_PER_SEC);

        if (ansDummy != ansSlow) {
            fprintf(stderr, "Slow's answer differs from dummy!\n");
            fprintf(stderr, "%s\n", ansDummy.c_str());
            fprintf(stderr, "%s\n", ansSlow.c_str());
            system("pause");
            exit(0);
        }

        unsigned oTime = clock();
        okay.init(test);
        string ansOkay;
        for (int i = 0; i < (int)test.queries.size(); i++) {
            if (test.queries[i].type == 1) { // Query
                ansOkay.push_back(okay.find(test.queries[i].val) ? 'Y' : 'N');
            } else { // Update
                okay.increment(
                    test.queries[i].row - 1, test.queries[i].col - 1, (int)test.queries[i].val);
            }
        }
        fprintf(stderr, "  >> okay's time: %.3lfs\n", (double)(clock() - oTime) / CLOCKS_PER_SEC);

        if (ansDummy != ansOkay) {
            fprintf(stderr, "Okay's answer differs from dummy!\n");
            fprintf(stderr, "%s\n", ansDummy.c_str());
            fprintf(stderr, "%s\n", ansOkay.c_str());
            system("pause");
            exit(0);
        }

        unsigned aTime = clock();
        author.init(test);
        string ansAuthor;
        for (int i = 0; i < (int)test.queries.size(); i++) {
            if (test.queries[i].type == 1) { // Query
                ansAuthor.push_back(author.find(test.queries[i].val) ? 'Y' : 'N');
            } else { // Update
                author.increment(
                    test.queries[i].row - 1, test.queries[i].col - 1, (int)test.queries[i].val);
            }
        }
        fprintf(stderr, "  >> author's time: %.3lfs\n", (double)(clock() - aTime) / CLOCKS_PER_SEC);
        
        if (ansDummy != ansAuthor) {
            fprintf(stderr, "Author's answer differs from dummy!\n");
            fprintf(stderr, "%s\n", ansDummy.c_str());
            fprintf(stderr, "%s\n", ansAuthor.c_str());
            system("pause");
            exit(0);
        }
    }
}

int n, m;
int a[MAX][MAX];
int tree[MAX][MAX];

void update(int x, int y, int val) {
    for (int x1 = x + 1; x1 < MAX; x1 += (x1 & -x1))
	    for (int y1 = y + 1; y1 < MAX; y1 += (y1 & -y1))
			tree[x1][y1] += val;
}

int query(int x, int y) {
	int ret = 0;
	for (int x1 = x + 1; x1 > 0; x1 -= (x1 & -x1))
	    for (int y1 = y + 1; y1 > 0; y1 -= (y1 & -y1))
	        ret += tree[x1][y1];
	return ret;
}

int getCell(int row, int col) {
    return a[row][col] + query(row, col);
}


void sampleInput() {
    Test test;
    test.board = vector < vector <int> >({
        {11, 13, 14, 17},
        {13, 15, 19, 21},
        {18, 20, 23, 42}
    });
    test.queries.push_back(Query(1, 42));
    test.queries.push_back(Query(1, 27));
    test.queries.push_back(Query(2, 2, 2, 4));
    test.queries.push_back(Query(1, 27));
    test.queries.push_back(Query(2, 1, 3, 7));
    test.queries.push_back(Query(1, 20));
    test.queries.push_back(Query(1, 32));
    test.queries.push_back(Query(1, 19));
    test.queries.push_back(Query(1, 42));

    tests.push_back(test);
}

void randomTest(int n_, int m_, int q_) {
    n = n_, m = m_;
    memset(tree, 0, sizeof(tree));

    a[0][0] = rnd() % 500000 + 1;
    for (int row = 1; row < n; row++)
        a[row][0] = a[row - 1][0] + rnd() % 500000 + 1;
    for (int col = 1; col < m; col++)
        a[0][col] = a[0][col - 1] + rnd() % 500000 + 1;
    
    for (int row = 1; row < n; row++)
        for (int col = 1; col < m; col++)
            a[row][col] = max(a[row - 1][col], a[row][col - 1]) + rnd() % 500000 + 1;
    
    Test test;
    for (int row = 0; row < n; row++) {
        test.board.push_back(vector <int>(a[row] + 0, a[row] + m));
    }

    // int qFreq = rnd() % 9; // Weighted queries and updates
    int qFreq = 4; // Evenly distributed queries and updates
    for (int i = 0; i < q_; i++) {
        if (rnd() % 10 <= qFreq) {
            // Query
            if (rnd() % 2 == 0) {
                // Random
                int num = min(2000000000, rnd() % (getCell(n - 1, m - 1) + 1000000) + 1);
                test.queries.push_back(Query(1, num));
            } else {
                // Existing
                int row = rnd() % n, col = rnd() % m;
                int num = getCell(row, col);
                test.queries.push_back(Query(1, num));
            }
        } else {
            // Update
            int row = rnd() % n, col = rnd() % m;
            while (row > n / 3 && rnd() % 2 == 0)
                row = rnd() % n;
            while (col > m / 3 && rnd() % 2 == 0)
                col = rnd() % m;
            int val = rnd() % 50000 + 1;
            test.queries.push_back(Query(2, row + 1, col + 1, val));
            update(row, col, val);
        }
    }
    tests.push_back(test);
}

void multMatrix(int n_, int m_, int q_) {
    n = n_, m = m_;
    for (int row = 0; row < n; row++)
        for (int col = 0; col < m; col++)
            a[row][col] = (row + 1) * (col + 1);
    memset(tree, 0, sizeof(tree));
    
    Test test;
    for (int row = 0; row < n; row++) {
        test.board.push_back(vector <int>(a[row] + 0, a[row] + m));
    }

    for (int i = 0; i < q_; i++) {
        if (i % 2 == 0) { // Query
            int row = rnd() % n, col = rnd() % m;
            int num = max(1, getCell(row, col) + 1 - rnd() % 3);
            test.queries.push_back(Query(1, num));
        } else { // Update
            int row = rnd() % n, col = rnd() % m, val = rnd() % 50000 + 1;
            test.queries.push_back(Query(2, row + 1, col + 1, val));
            update(row, col, val);
        }
    }
    tests.push_back(test);
}

void plusOneMatrix(int n_, int m_, int q_) {
    n = n_, m = m_;
    for (int row = 0; row < n; row++)
        for (int col = 0; col < m; col++)
            a[row][col] = row + col + 1;
    memset(tree, 0, sizeof(tree));
    
    Test test;
    for (int row = 0; row < n; row++) {
        test.board.push_back(vector <int>(a[row], a[row] + m));
    }

    for (int i = 0; i < q_; i++) {
        if (i % 2 == 0) { // Query
            int row = rnd() % n, col = rnd() % m;
            int num = max(1, getCell(row, col) + 1 - rnd() % 3);
            test.queries.push_back(Query(1, num));
        } else { // Update
            int row = rnd() % n, col = rnd() % m, val = rnd() % 50000 + 1;
            test.queries.push_back(Query(2, row + 1, col + 1, val));
            update(row, col, val);
        }
    }
    tests.push_back(test);
}

const int MIN_N = 1, MAX_N = 1000;
const int MIN_M = 1, MAX_M = 1000;
const int MIN_A = 1, MAX_A = 1000000000;
const int MIN_Q = 1, MAX_Q = 20000;
const int MIN_X = 1, MAX_X = 2000000000;
const int MIN_Y = 1, MAX_Y = 50000;

void sanityCheck(Test& test, int testNum) {
    if ((int)test.board.size() < MIN_N || (int)test.board.size() > MAX_N) {
        fprintf(stderr, "ERROR in test %d: N is out of range (%d with range [%d, %d])!\n",
                testNum, (int)test.board.size(), MIN_N, MAX_N);
    }
    if ((int)test.board[0].size() < MIN_M || (int)test.board[0].size() > MAX_M) {
        fprintf(stderr, "ERROR in test %d: M is out of range (%d with range [%d, %d])!\n",
                testNum, (int)test.board[0].size(), MIN_M, MAX_M);
    }
    for (int row = 0; row < (int)test.board.size(); row++) {
        for (int col = 0; col < (int)test.board[row].size(); col++) {
            if (test.board[row][col] < MIN_A || test.board[row][col] > MAX_A) {
                fprintf(stderr, "ERROR in test %d: Aij is out of range (%d with range [%d, %d])!\n",
                        testNum, test.board[row][col], MIN_A, MAX_A);
            }
        }
    }
    if ((int)test.queries.size() < MIN_Q || (int)test.queries.size() > MAX_Q) {
        fprintf(stderr, "ERROR in test %d: Q is out of range (%d with range [%d, %d])!\n",
                testNum, (int)test.queries.size(), MIN_Q, MAX_Q);
    }
    for (int i = 0; i < (int)test.queries.size(); i++) {
        Query& q = test.queries[i];
        if (q.type != 1 && q.type != 2) {
            fprintf(stderr, "ERROR in test %d: query type is invalid (%d but must be 1 or 2)!\n",
                    testNum, q.type);
        }
        if (q.type == 1) {
            if (q.val < MIN_X || q.val > MAX_X) {
                fprintf(stderr, "ERROR in test %d: type 1 query value is out of range (%d with range [%d, %d])!\n",
                        testNum, q.val, MIN_X, MAX_X);
            }
        } else {
            if (q.row < 1 || q.row > (int)test.board.size()) {
                fprintf(stderr, "ERROR in test %d: type 2 query row is out of range (%d with range [%d, %d])!\n",
                        testNum, q.row, 1, (int)test.board.size());
            }
            if (q.col < 1 || q.col > (int)test.board[0].size()) {
                fprintf(stderr, "ERROR in test %d: type 2 query column is out of range (%d with range [%d, %d])!\n",
                        testNum, q.col, 1, (int)test.board[0].size());
            }
            if (q.val < MIN_Y || q.val > MAX_Y) {
                fprintf(stderr, "ERROR in test %d: type 2 query value is invalid (%d but must be in range [%d, %d])!\n",
                        testNum, q.val, MIN_Y, MAX_Y);
            }
        }
    }
}

void printTests() {
    Author solution;
    for (int t = 0; t < (int)tests.size(); t++) {
        fprintf(stderr, "Printing testcase %d out of %d...\n", t, (int)tests.size() - 1);
        
        Test& test = tests[t];
        sanityCheck(test, t);
        solution.init(test);

        char inpFile[32], solFile[32];
        sprintf(inpFile, "MatrixSearch.%02d.in", t);
        sprintf(solFile, "MatrixSearch.%02d.sol", t);
        FILE* inp = fopen(inpFile, "wt");
        FILE* sol = fopen(solFile, "wt");

        fprintf(inp, "%d %d\n", (int)test.board.size(), (int)test.board[0].size());
        for (int row = 0; row < (int)test.board.size(); row++)
            for (int col = 0; col < (int)test.board[row].size(); col++)
                fprintf(inp, "%d%c", test.board[row][col],
                        col + 1 == (int)test.board[row].size() ? '\n' : ' ');
        
        fprintf(inp, "%d\n", (int)test.queries.size());
        for (int q = 0; q < (int)test.queries.size(); q++) {
            fprintf(inp, "%d ", test.queries[q].type);
            if (test.queries[q].type == 1) {
                fprintf(inp, "%d\n", test.queries[q].val);
                fprintf(sol, "%c\n", solution.find(test.queries[q].val) ? 'Y' : 'N');
            } else {
                fprintf(inp, "%d %d %d\n", test.queries[q].row, test.queries[q].col, test.queries[q].val);
                solution.increment(test.queries[q].row - 1, test.queries[q].col - 1, test.queries[q].val);
            }
        }
        fclose(inp), fclose(sol);
    }
}

int main(void) {
    sampleInput();

    // O(Q * N * M) => 30 points
    randomTest(10, 10, 10);
    randomTest(2, 100, 200);
    randomTest(100, 2, 200);
    randomTest(204, 198, 1000);
    randomTest(404, 493, 5000);
    randomTest(1000, 1000, 500);
    
    // O(Q * (N + M) * log^2(N)) => 30 points
    randomTest(400, 400, 20000);
    randomTest(500, 500, 20000);
    randomTest(600, 600, 15000);
    randomTest(999, 786, 8000);
    multMatrix(1000, 1000, 9000);
    plusOneMatrix(1000, 1000, 10000);
    
    // O(Q * (N + M)) => 40 points
    randomTest(666, 666, 16161); // Maybe in
    randomTest(777, 777, 17171);
    randomTest(888, 888, 18181);
    randomTest(999, 999, 19191);
    randomTest(1000, 1000, 10000); // Maybe in
    randomTest(1000, 1000, 20000);
    multMatrix(1000, 1000, 20000);
    plusOneMatrix(1000, 1000, 20000);
    
    testSolutions();
    printTests();
        
    return 0;
}
